#!/bin/bash
# $Date:: 2015-05-08 16:55:35 +0900#$
# $Revision: 10745 $
# Usage: osvd_collect_log.sh {email | full}

date=`date +'%y%m%d_%H%M%S'`
name="osvdlog_${date}"

function usage_error() {
    echo "usage: ./osvd_collect_log.sh <mode> [ <output path> ] (mode=email, full)"
    exit 1
}

function copy_log() {
    target=$1
    output=$2
    if [ -e "$target" ]; then
        echo "copy $target => $output"
        tar -jcf "${output}.tar.bz2" $target 2>/dev/null
    fi
}

function copy_logs() {
    target=$1
    output=$2
    
    ls $target >/dev/null 2>&1
    if [ $? -eq 0 ]; then
        echo "copy $target => $output"
        tar -jcf "${output}.tar.bz2" $target 2>/dev/null
    fi
}

function get_cmdresult() {
    cmdname=$1
    cmd=$2
    output=$3

    which $cmdname >/dev/null 2>&1
    if [ $? -eq 0 ]; then
        echo "exec $cmd => $output"
        $cmd >$output 2>&1
    fi
}

# init
mode=$1
output_path=${2%/}

if [ -z "$output_path" ]; then
    output_path="."
elif [ ! -d $output_path ]; then
    echo "Invalid output path was specified : $output_path"
    exit 1
fi

dir="${output_path}/${name}"

#check mode
if [ -z "${mode}" ]; then
    usage_error
elif [ "${mode}" == "email" ]; then
    arcfile="${output_path}/${name}_email.tar.bz2"
elif [ "${mode}" == "full" ]; then
    arcfile="${output_path}/${name}_full.tar.bz2"
else
    usage_error
fi

mkdir $dir

#/var/log 
copy_log /var/log/messages $dir/messages_log

if [ "${mode}" == "email" ]; then
    copy_log /var/log/cinder/volume.log $dir/cinder_volume_log
    copy_log /var/log/nova/nova-compute.log $dir/nova_compute_log
    copy_logs "/etc/cinder/*.xml" $dir/cinder_xml
elif [ "${mode}" == "full" ]; then
    copy_log /var/log/cinder $dir/cinder_log
    copy_log /var/log/nova $dir/nova_log
fi

#/etc
copy_log /etc/cinder $dir/cinder_conf
copy_log /etc/nova $dir/nova_conf
copy_log /etc/multipath.conf $dir/multipath_conf
copy_log /etc/iscsi $dir/iscsi_conf

if [ "${mode}" == "full" ]; then
    copy_log /var/lib/iscsi $dir/lib_iscsi
fi

#command result
get_cmdresult ifconfig "ifconfig" $dir/ifconfig.txt
get_cmdresult ls "ls /dev/sd* -la" $dir/sdx.txt
get_cmdresult ls "ls /dev/disk/by-id -la" $dir/by-id.txt
get_cmdresult ls "ls /dev/disk/by-path -la" $dir/by-path.txt
get_cmdresult ls "ls /dev/disk/by-uuid -la" $dir/by-uuid.txt
get_cmdresult multipath "multipath -ll" $dir/multipath_ll.txt
get_cmdresult iscsiadm "iscsiadm -m session" $dir/iscsiadm_m_session.txt
get_cmdresult iscsiadm "iscsiadm -m node" $dir/iscsiadm_m_node.txt

#archive temporary file
tar -jcvf $arcfile $dir
rm -rf $dir

#check the size of arcive file
arcsize=`wc -c < ${arcfile}`

if [ "${mode}" == "email" ]; then
    if [ ${arcsize} -ge 5242880 ]; then
        split -b 5m -d "${arcfile}" "${arcfile}."
        rm -f ${arcfile}
    fi
fi
