#!/bin/bash
########################################################################
#
# @(#)osvd_install.sh
#
# Copyright FUJITSU LIMITED 2022
#
# @version 1.6.1.2 (2022.01)
#
########################################################################

#---------------
# Definition
#---------------
PRODUCT_NAME="ETERNUS OpenStack VolumeDriver"
VERSION="1.6.1.2"
COPYRIGHT="Copyright FUJITSU LIMITED 2022"

SH_NAME=$(basename $0)
CAT_CMD="/bin/cat"
CP_CMD="/bin/cp"
GREP_CMD="/bin/grep"
LS_CMD="/bin/ls"
RM_CMD="/bin/rm"
SED_CMD="/bin/sed"
AWK_CMD="/usr/bin/awk"
CUT_CMD="/usr/bin/cut"
FIND_CMD="/usr/bin/find"
TR_CMD="/usr/bin/tr"
WHOAMI_CMD="/usr/bin/whoami"
MKDIR_CMD="/usr/bin/mkdir"
SSHKEYGEN_CMD="/usr/bin/ssh-keygen"
RPM_CMD="/usr/bin/rpm"
WC_CMD="/usr/bin/wc"
CHOWN_CMD="/usr/bin/chown"
RPM_NAME="pike_osvd-1.6.1.2-1.el7.x86_64"

. ./osvd_install.def
CINDER_CONF="/etc/cinder/cinder.conf"

CINDER_LIB=""
ENABLED_BACKENDS=""
VOLUME_BACKEND_NAME=""
MAPPING_FILE_PATH=""
DB_USERNAME=""
DB_PASSWORD=""

#---------------
# Functions
#---------------

# print with level
PR()
{
	msg=$1
	level=$2
	if [ "${level}" = "" ]
	then
		level="INFO"
		echo "${SH_NAME}: ${level}: ${msg}"
	else
		echo "${SH_NAME}: ${level}: ${msg}" >&2
	fi
}

# print product information
PR_PRODUCT_INFO()
{
	echo ""
	echo ${PRODUCT_NAME}
	echo ${VERSION}
	echo ${COPYRIGHT}
	echo ""
}

# print error and usage
PR_ERROR_USAGE()
{
	echo "Usage: ./osvd_install.sh {install | update | uninstall} [ install_config_file ]"
	exit 1
}

# print title
TITLE()
{
	echo ""
	echo "====================================="
	echo "  $1"
	echo "====================================="
}

# find install directory
FIND_INSTDIR()
{
	CINDER_DIR=`python -c 'import cinder; print(cinder.__file__)'`
	inst_dir=`echo $CINDER_DIR | ${SED_CMD} -e 's/__init__.pyc/volume\/drivers\//g'`
	if [ ! -d "${inst_dir}" ]; then
        PR "Can not find install directory." "ERROR"
		exit 1
	fi
	echo ${inst_dir}
}

#---------------
# INSTALL SubFunctions
#---------------

# check installed
CHECK_INSTALLED()
{
	local inst_dir=$1
	local inst_cfg=$2

	TITLE "check volume driver"

	if [ -d ${inst_dir}fujitsu/eternus_dx ]
	then
		PR "This driver has already been installed."
		if [ -z "${inst_cfg}" ]; then
			YES_NO "Overwrite the driver?"
			if [ $? -ne 0 ]; then
				PR "The installation is ended."
				PR "Please execute \"${SH_NAME} uninstall\" to reinstall this driver."
				exit 0
			else
				DELETE_FILE ${inst_dir}
			fi
		else
			DELETE_FILE ${inst_dir}
		fi
	fi
}

# put files
PUT_FILE()
{
	inst_dir=$1
	TITLE "Put driver files"
	echo "Copy driver files to ${inst_dir}"
	${RPM_CMD} -U "${RPM_NAME}.rpm" --force
	if [ $? -ne 0 ]; then
		PR "install rpm FAILED!!" "ERROR"
		${CP_CMD} -rpf fujitsu ${inst_dir}
		if [ $? -ne 0 ]; then
			PR "install FAILED!!" "ERROR"
			exit 1
		fi
	fi
	if [ "${inst_dir}" != "/usr/lib/python2.7/site-packages/cinder/volume/drivers/" ]; then
		${CP_CMD} -rpf fujitsu ${inst_dir}
		if [ $? -ne 0 ]; then
		    PR "install FAILED!!" "ERROR"
		    exit 1
		fi
	fi	
}

#---------------
# UPDATE_CONFIG SubFunctions
#---------------
# select protocol
SELECT_PROTOCOL()
{
	TITLE "select protocol"
	echo "1) FibreChannel"
	echo "2) iSCSI"
	echo -n "Which way of FibreChannel and iSCSI do you use?[1 or 2]: "
	read -r protocol
	while [ "${protocol}" != "1" ] && [ "${protocol}" != "2" ]
	do
		echo -n "Please select 1 or 2. "
		read -r protocol
	done

	case "${protocol}" in
		"1") PR "protocol is FibreChannel";;
		"2") PR "protocol is iSCSI";;
	esac
	return ${protocol}
}

# select http/https
SELECT_HTTP_OR_HTTPS()
{
	TITLE "select ETERNUS port number for SMIS"
	echo "1) http port: 5988"
	echo "2) https port: 5989"
	echo -n "Which way of http and https do you use?[1 or 2]: "
	read -r port
	while [ "${port}" != "1" ] && [ "${port}" != "2" ]
	do
		echo -n "Please select 1 or 2. "
		read -r port
	done

	case "${port}" in
		"1") PR "http's port 5988 has been selected";;
		"2") PR "https's port 5989 has been selected";;
	esac

	return ${port}
}

# yes or not
YES_NO()
{
	while true
	do
		echo -n "$1 [yes/no]: "
		read -r answer
		case ${answer} in
			yes)
				return 0;;
			no)
				return 1;;
			*)
				echo "please input yes or no.";;
		esac
	done
}

# output information to cinder.conf
OUTPUT_CINDER_CONF()
{
	local section=$1
	local conf=$2
	local params=${*:3}
	local driver_path=""
	local i=0
	local line=""
	local line1=""
	local line2=""
	local line3=""

	${GREP_CMD} -i "\[${section}\]" ${CINDER_CONF} >/dev/null
	if [ $? -eq 0 ]; then
		echo "SECTION: ${section} has already existed, Skip update"
		return 1
	fi

	if [ ${prtcl} == 'fc' ]; then
		driver_path="volume_driver=cinder.volume.drivers.fujitsu.eternus_dx.eternus_dx_fc.FJDXFCDriver"
	elif [ ${prtcl} == 'iscsi' ]; then
		driver_path="volume_driver=cinder.volume.drivers.fujitsu.eternus_dx.eternus_dx_iscsi.FJDXISCSIDriver"
	fi

	drvconfig_path="cinder_eternus_config_file=${conf}"

	# output to cinder.conf
	echo "[${section}]" >> ${CINDER_CONF}
	echo "${driver_path}" >> ${CINDER_CONF}
	echo "${drvconfig_path}" >> ${CINDER_CONF}
	if [ ! -z "${MAPPING_FILE_PATH}" ]; then
		echo "fujitsu_port_host_mapping_file=${MAPPING_FILE_PATH}" >>${CINDER_CONF}
		MAPPING_FILE_PATH=""
	fi

	line1=`${SED_CMD} -n '/^\s*enabled_backends\s*=/=' ${CINDER_CONF} | ${SED_CMD} -n "1"p`
	line2=`${SED_CMD} -n '/^#enabled_backends\s*=/=' ${CINDER_CONF} | ${SED_CMD} -n "1"p`
	line3=`${SED_CMD} -n '/#.*enabled_backends\s*/=' ${CINDER_CONF} | ${SED_CMD} -n "1"p`
	line4=`${SED_CMD} -n '/\s*\[DEFAULT\]\s*/=' ${CINDER_CONF} | ${SED_CMD} -n "1"p`

	if [ ! -z "${ENABLED_BACKENDS}" ];then
		ENABLED_BACKENDS=`echo "${ENABLED_BACKENDS}" | ${SED_CMD} 's/\s*,\s*/,/g'`
		ENABLED_BACKENDS=`echo "${ENABLED_BACKENDS}" | ${SED_CMD} 's/\t/\[tab\]/g'`
		ENABLED_BACKENDS=`echo "${ENABLED_BACKENDS}" | ${SED_CMD} 's/\s/\[space\]/g'`
		if [ ! -z "${line1}" ];then
			${SED_CMD} -i ''${line1}'s/enabled_backends\s*=.*/enabled_backends='${ENABLED_BACKENDS}'/' ${CINDER_CONF}
		elif [ ! -z "${line2}" ];then
			${SED_CMD} -i ''${line2}'a\enabled_backends='${ENABLED_BACKENDS}'\n' ${CINDER_CONF}
		elif [ ! -z "${line3}" ];then
			${SED_CMD} -i ''${line3}'a\enabled_backends='${ENABLED_BACKENDS}'\n' ${CINDER_CONF}
		elif [ ! -z "${line4}" ];then
			${SED_CMD} -i ''${line4}'a\\nenabled_backends='${ENABLED_BACKENDS}'\n' ${CINDER_CONF}
		else
			echo "ERROR: enabled_backends is set failed!"
			exit 1
		fi
		line5=`${SED_CMD} -n '/'${ENABLED_BACKENDS}'/=' ${CINDER_CONF} | ${SED_CMD} -n "1"p`
		${SED_CMD} -i ''${line5}'s/\[space\]/\ /g' ${CINDER_CONF}
		${SED_CMD} -i ''${line5}'s/\[tab\]/\t/g' ${CINDER_CONF}
		ENABLED_BACKENDS=""
	fi

	if [ ! -z "${VOLUME_BACKEND_NAME}" ];then
		echo volume_backend_name="${VOLUME_BACKEND_NAME}" >>${CINDER_CONF}
		VOLUME_BACKEND_NAME=""
	fi
	for line in ${params[@]}
	do
		echo ${line} >>${CINDER_CONF}
	done

	return 0
}

# create and output information to cinder_driver_config_file
OUTPUT_DRIVER_CONF()
{
	local section=$1
	local conf=$2
	local ip=$3
	local port=$4
	local user=$5
	local passwd=$6
	local poollist=$7
	local snappoollist=$8

	${CAT_CMD} <<_EOT_ >"${conf}"
<?xml version='1.0' encoding='UTF-8'?>
<FUJITSU>
<EternusIP>${ip}</EternusIP>
<EternusPort>${port}</EternusPort>
<EternusUser>${user}</EternusUser>
<EternusPassword>${passwd}</EternusPassword>
_EOT_

	TMPIFS=${IFS}
	IFS=','
	for pool in ${poollist}
	do
		echo "<EternusPool>${pool}</EternusPool>" >> "${conf}"
	done

	for snappool in ${snappoollist}
	do
		echo "<EternusSnapPool>${snappool}</EternusSnapPool>" >> "${conf}"
	done
	echo "</FUJITSU>" >> "${conf}"

	if [ $? -ne 0 ]; then
		# failed to create specified driver configuration file, return failure
		echo "Failed to create ${conf}"
		exit 1
	fi

	IFS=${TMPIFS}

	if [ -f "${conf}" ]; then
		return 0
	else
		# failed to create specified driver configuration file, return failure
		echo "Failed to create ${conf}"
		exit 1
	fi
}

# edit config files with manual
EDIT_CONFIG_MANUAL()
{
	local ans=1

	while [ ${ans} -eq 1 ]
	do
		local section=""
		local conf=""
		local ip=""
		local port=5988
		local user=""
		local passwd=""
		local pool=""
		local snappool=""
		local mapping_file=""
		local volume_backend_name=""
		local verif="false"
		local ca_path=""

		TITLE "input driver configuration"

		while [ -z "${section}" ]
		do
			echo -n "section: "
			read -r section
		done

		while [ -z "${conf}" ]
		do
			echo -n "driver configuration path (.xml): "
			read -r conf
		done

		# check driver config file exsistence
		while [ -f "${conf}" ]
		do
			echo "SECTION: ${section}, specified driver configuration file (${conf}) has already existed"
			YES_NO "Overwrite the file (${conf})?"
			if [ $? -ne 0 ]; then
				conf=""
				while [ -z "${conf}" ]
				do
					echo -n "please re-enter driver configuration path (.xml): "
					read -r conf
				done
			else
				break
			fi
		done

		echo "input host affinity configuration file path, if you don't want to set please enter to skip"
		echo -n "host affinity configuration file path: "
		read -r mapping_file
		if [ ! -z "${mapping_file}" ]; then
			MAPPING_FILE_PATH="${mapping_file}"
		    # check host affinity configuration file exsistence
		    if [ ! -f "${mapping_file}" ]; then
		    	echo "Warning!!!"
		    	echo "SECTION: ${section}, specified host affinity configuration file (${mapping_file}) has not existed!!"
		    	echo "Please remeber to create it!!!"
		    fi
		fi

		echo "input volume backend name, if you don't want to set please enter to skip"
		echo -n "volume_backend_name:"
		read -r volume_backend_name
		if [ ! -z "${volume_backend_name}" ]; then
			VOLUME_BACKEND_NAME="${volume_backend_name}"
		fi

		SELECT_PROTOCOL
		if [ $? -eq 1 ]; then #fc
			prtcl='fc'
		else
			prtcl='iscsi'
		fi

		while [ -z "${ip}" ]
		do
			echo -n "1)ETERNUS ip address for SMIS: "
			read -r ip
			ip=`echo ${ip} | ${TR_CMD} -d "' '|'\t'"`
		done

		# port
		echo -n "2)ETERNUS port number for SMIS: "
		SELECT_HTTP_OR_HTTPS
		if [ $? -eq 2 ]; then #HTTP
			port='5989'
			YES_NO "Do you want to verify the SSL certficate?"
			if [ $? -ne 0 ]; then
				PR "You has selected to ignore verifying the SSL certficate."
			else
				verif="true"
				echo -n "Please input CA Cert path:(default:/etc/pki/tls/certs/ca-bundle.trust.crt):"
				read -r ca_path
				ca_path=`echo "${ca_path}" | ${SED_CMD} 's/\t//g'`
				if [ -z "${ca_path}" ]; then
					ca_path="/etc/pki/tls/certs/ca-bundle.trust.crt"
				fi
				if [ ! -e "${ca_path}" ]; then
					echo "Warning!!!"
					echo "SECTION: ${section}, specified system path with ca certificates (${ca_path}) has not existed!!"
					echo "Please remeber to create it!!!"
				fi
			fi
		else
			port='5988'
		fi

		# usr
		while [ -z "${user}" ]
		do
			echo -n "3)ETERNUS username for SMIS: "
			read -r user
			user=`echo ${user} | ${TR_CMD} -d "' '|'\t'"`
		done

		# pass
		while [ -z "${passwd}" ]
		do
			echo -n "4)ETERNUS password for SMIS: "
			read -r passwd
			passwd=`echo ${passwd} | ${TR_CMD} -d "' '|'\t'"`
		done

		# pool
		while [ -z "${pool}" ]
		do
			echo -n "5)ETERNUS poolname (e.g. Pool01,Pool02,Pool03): "
			read -r pool
			pool=`echo "${pool}" | ${SED_CMD} 's/\s*,\s*/,/g'`
			pool=`echo "${pool}" | ${SED_CMD} 's/\t//g'`
		done

		# snappool
		echo -n "6)ETERNUS snappoolname (e.g. Snappool01,Snappool02,Snappool03): "
		read -r snappool
		if [ -z "${snappool}" ]; then
			snappool=`echo "${pool}" | ${CUT_CMD} -d ',' -f 1`
		elif [ ! -z "${snappool}" ]; then
		    snappool=`echo "${snappool}" | ${SED_CMD} 's/\s*,\s*/,/g'`
			snappool=`echo "${snappool}" | ${SED_CMD} 's/\t//g'`
		fi

		echo ""
		echo "1)IP      : ${ip}"
		echo "2)PORT    : ${port}"
		echo "3)USR     : ${user}"
		echo "4)PASS    : ${passwd}"
		echo "5)POOL    : ${pool}"
		echo "6)SNAPPOOL: ${snappool}"
		YES_NO "Are you sure?"
		ans=$?
	done

	# output informations to cinder.conf
	if [[ "${verif}" == "false" ]]; then
		OUTPUT_CINDER_CONF "${section}" "${conf}"
	else
		verification="fujitsu_verification=True"
		if [[ ca_file_path == "/etc/pki/tls/certs/ca-bundle.trust.crt" ]]; then
			OUTPUT_CINDER_CONF "${section}" "${conf}" "${verification}"
		else
			ca_file_path="fujitsu_ca_certs_path=${ca_path}"
			OUTPUT_CINDER_CONF "${section}" "${conf}" "${verification}" "${ca_file_path}"
		fi	
	fi

	if [ $? -ne 0 ]; then
		return
	fi

	# output information to driver config file(xml)
	OUTPUT_DRIVER_CONF "${section}" "${conf}" "${ip}" "${port}" "${user}" "${passwd}" "${pool}" "${snappool}"
	if [ $? -ne 0 ]; then
		return
	fi
}

# update config with manual
UPDATE_CONFIG_MANUAL()
{
	local loop_stop=0
	local ans=1
	local enabled_backends=""

	while [ ${ans} -eq 1 ]
	do
		echo "input enabled backends, if you don't want to change please enter to skip"
		echo -n "enabled backends:"
		read -r enabled_backends
		if [ ! -z "${enabled_backends}" ]; then
			echo "ENABLED_BACKENDS : ${enabled_backends}"
		fi
		YES_NO "Are you sure?"
		ans=$?
	done
	if [ ! -z "${enabled_backends}" ]; then
		ENABLED_BACKENDS=${enabled_backends}
	fi

	while [ ${loop_stop} -ne 1 ]
	do
		EDIT_CONFIG_MANUAL
		YES_NO "Continue to add configuration?"
		loop_stop=$?
	done
}

# edit config files silently
EDIT_CONFIG_SILENT()
{
	local section=$1
	local params=${*:2}
	local enabled="false"
	local prtcl=""
	local conf=""
	local mapping_file=""
    local volume_backend_name=""
	local enabled_backends=""
	local ip=""
	local port=5988
	local user=""
	local passwd=""
	local pool=""
	local snappool=""
	local verif="false"
	local ca_path="/etc/pki/tls/certs/ca-bundle.trust.crt"
	local others=()
	local i=0

	for param in ${params[@]}
	do
		key=`echo $param | ${CUT_CMD} -d '=' -f 1`
		value=`echo $param | ${CUT_CMD} -d '=' -f 2-`
		case ${key} in
			${STORAGE_ENABLED_KEY})  enabled=${value,,};;
			${STORAGE_PRTCL_KEY})    prtcl=${value,,};;
			${STORAGE_CONF_KEY})     conf=${value};;
			${STORAGE_MAPPING_KEY})  mapping_file=${value};;
			${STORAGE_IP_KEY})       ip=${value};;
			${STORAGE_PORT_KEY})     port=${value};;
			${STORAGE_USR_KEY})      user=${value};;
			${STORAGE_PASSWD_KEY})   passwd=${value};;
			${STORAGE_POOL_KEY})     pool=${value};;
			${STORAGE_SNAPPOOL_KEY}) snappool=${value};;
			${STORAGE_BACKEND_KEY})  volume_backend_name=${value};;
			${STORAGE_BACKEND_KEY2}) enabled_backends=${value};;
			${STORAGE_HTTPS_VERIF})  verif=${value};;
			${STORAGE_HTTPS_CA_PATH}) ca_path=${value};;
            ${STORAGE_DB_USERNAME})  db_username=${value};;
            ${STORAGE_DB_PASSWORD})  db_password=${value};;
			*)                       others+=(${param});;
		esac
	done

	if [ "${section}" == 'DEFAULT' ]; then
		if [ ! -z "${enabled_backends}" ]; then
			ENABLED_BACKENDS=${enabled_backends}
		fi
        if [ ! -z "${db_username}" ]; then
            DB_USERNAME=${db_username}
        fi
        if [ ! -z "${db_password}" ]; then
            DB_PASSWORD=${db_password}
        fi
		return
	fi

	if [ ! -z "${volume_backend_name}" ]; then
		VOLUME_BACKEND_NAME=${volume_backend_name}
	fi

	if [ "${enabled}" != 'true' ]; then
		echo "SECTION: ${section}, Skip"
		return
	fi

	if [ -z "${prtcl}" ] || [ -z "${conf}" ] || [ -z "${ip}" ] || [ -z "${port}" ] || \
		[ -z "${user}" ] || [ -z "${passwd}" ] || [ -z "${pool}" ]; then
		echo "SECTION: ${section}, Invalid Parameter. Please check each parameter."
		echo "Protocol : ${prtcl}"
		echo "Driver Configuration File : ${conf}"
		echo "IP Address: ${ip}"
		echo "PortNo: ${port}"
		echo "User: ${user}"
		echo "Password: ${passwd//?/'*'}"
		echo "PoolName: ${pool}"
		exit 1
	fi

	# check protocol type
	if [ "${prtcl}" != 'fc' ] && [ "${prtcl}" != 'iscsi' ]; then
		echo "SECTION: ${section}, Invalid Protocol: ${prtcl}"
        exit 1
	fi

	# check host affinity configuration file exsistence
	if [ ! -z "${mapping_file}" ]; then
	    MAPPING_FILE_PATH="${mapping_file}"
	    if [ ! -f "${mapping_file}" ]; then
		    echo "Warning!!!"
		    echo "SECTION: ${section}, specified host affinity configuration file (${mapping_file}) has not existed!!"
		    echo "Please remeber to create it!!!"
		fi
    fi

	# when snappool is null, set poolname to it
	if [ -z "${snappool}" ]; then
		snappool=`echo ${pool} | ${CUT_CMD} -d ',' -f 1`
	fi

	# output informations to cinder.conf
	if [ "${port}" == "5989" ]; then
		if [ "${verif}" != 'true' ]; then
			OUTPUT_CINDER_CONF "${section}" "${conf}" "${others[@]}"
		else
			if [ ! -f "${ca_path}" ]; then
				echo "Warning!!!"
				echo "SECTION: ${section}, specified system path with ca certificates (${ca_path}) has not existed!!"
				echo "Please remeber to create it!!!"
			fi
			verification="fujitsu_verification=True"
			ca_file_path="fujitsu_ca_certs_path=${ca_path}"
			OUTPUT_CINDER_CONF "${section}" "${conf}" "${verification}" "${ca_file_path}" "${others[@]}"			
		fi
	else
		OUTPUT_CINDER_CONF "${section}" "${conf}" "${others[@]}"
	fi

	if [ $? -ne 0 ]; then
		return
	fi

	# check driver config file exsistence
	if [ -f "${conf}" ]; then
		echo "SECTION: ${section}, specified driver configuration file (${conf}) has already existed, Skip"
		return
	fi

	# output information to driver config file(xml)
	OUTPUT_DRIVER_CONF "${section}" "${conf}" "${ip}" "${port}" "${user}" "${passwd}" "${pool}" "${snappool}"
	if [ $? -ne 0 ]; then
		return
	fi

	echo "SECTION: ${section}, Configure complete to ${CINDER_CONF} and ${conf}."
}

# update config silently
UPDATE_CONFIG_SILENT()
{
	local inst_cfg=$1
	local section=""
	local params=()
	local line=""

	OLDIFS=${IFS}
	IFS=$'\n'
	for rawline in `${CAT_CMD} ${inst_cfg}`
	do
		line=`echo ${rawline} | ${TR_CMD} -d "' '|'\t'"`

		# if line is null or comment, then skip
		echo ${line} | ${GREP_CMD} "#\|^$">/dev/null
		if [ $? -eq 0 ]; then
			continue
		fi

		## if line is section line, then update section information
		echo ${line} | ${GREP_CMD} "^\[" | ${GREP_CMD} "\]$" >/dev/null
		if [ $? -eq 0 ]; then
			# flush previous section information
			if [ -n "${section}" ]; then
				EDIT_CONFIG_SILENT "${section}" "${params[@]}"
			fi

			# initialize section information
			section=`echo ${line} | ${SED_CMD} 's/^\[\|\]$//g'`
			params=()
		else
			params+=($line)
		fi

	done

	# flush previous section information
	if [ -n "${section}" ]; then
		EDIT_CONFIG_SILENT "${section}" "${params[@]}"
	fi
	IFS=${OLDIFS}
}

CHECK_PYOPENSSL()
{
	local pyOpenSSL_version=`python -c 'from OpenSSL import version;print(version.__version__);' 2>>/dev/null`
	local error_version='17.3.0'
	if [ $pyOpenSSL_version = $error_version ]
	then
		PR 'The pyOpenSSL package version is 17.3.0, Fujitsu driver may fail to start. Please upgrade the package!' 'WARNING'
	fi
}

#---------------
# DELETE SubFunctions
#---------------

# delete files
DELETE_FILE()
{
	inst_dir=$1
	${RPM_CMD} -qa | ${GREP_CMD} ${RPM_NAME} | xargs ${RPM_CMD} -e
	touch ${inst_dir}fujitsu/__init__.py
	TITLE "delete driver files"
	echo "remove fujitsu from ${inst_dir}"
	${RM_CMD} -r ${inst_dir}fujitsu/eternus_dx 2>/dev/null
}

# INSTALL
INSTALL()
{
	local inst_cfg=$1
	local inst_dir=""

	TITLE "Install Start"

	if [ -z "$1" ]; then
		echo "Install start with interactive input"
	else
		echo "Install start with ${inst_cfg}"
	fi

	inst_dir=`FIND_INSTDIR 2>/dev/null`
	if [ $? -ne 0 ]; then
		PR "Cannot find directory for cinder volume drivers" "ERROR"
		exit 1
	fi

	CHECK_INSTALLED "${inst_dir}" "${inst_cfg}"
	PUT_FILE "${inst_dir}"
	UPDATE_CONFIG "${inst_cfg}"

# rootwrap
	TITLE "set rootwrap for mkdir"
	if [ ! -d /etc/cinder/rootwrap.d ]
	then
		PR "create directory[/etc/cinder/rootwrap.d]"
		${MKDIR_CMD} /etc/cinder/rootwrap.d
	fi

	if [ -f /etc/cinder/rootwrap.d/volume.filters ]
	then
		count=`${GREP_CMD} -c mkdir /etc/cinder/rootwrap.d/volume.filters`
		if [ ${count} -eq 0 ]
		then
			PR "add mkdir [/etc/cinder/rootwrap.d/volume.filters]"
			echo "mkdir: CommandFilter, mkdir, root" >> /etc/cinder/rootwrap.d/volume.filters
		fi
	else
		PR "create file [/etc/cinder/rootwrap.d/volume.filters]"
		echo "[Filters]" > /etc/cinder/rootwrap.d/volume.filters
		echo "mkdir: CommandFilter, mkdir, root" >> /etc/cinder/rootwrap.d/volume.filters
	fi

	if [ -z ${CINDER_LIB} ]; then
		state_path=`${GREP_CMD} '^\s*state_path\s*=' ${CINDER_CONF} 2>/dev/null | ${SED_CMD} -n "1"p | ${CUT_CMD} -d'=' -f2 | ${SED_CMD} 's/\s*//g'`
		if [ -z ${state_path} ]; then
			state_path=`${GREP_CMD} '^\s*#\s*state_path\s*=' ${CINDER_CONF} | ${SED_CMD} -n "1"p | ${CUT_CMD} -d'=' -f2 | ${SED_CMD} 's/\s*//g'`
			if [ -z ${state_path} ]; then
				state_path='/var/lib/cinder'
			fi
		fi
		CINDER_LIB=${state_path}
	fi

    # create ssh key
	TITLE "create ssh key for eternus"
	if [ ! -f /root/.ssh/eternus ]
	then
		PR "create ssh key"
		${SSHKEYGEN_CMD} -t rsa -N "" -f /root/.ssh/eternus
	fi
	PR "create eternus.ietf"
	${SSHKEYGEN_CMD} -e -f /root/.ssh/eternus.pub > ./eternus.ietf
    PR "copy private key"
    ${CP_CMD} -f -p /root/.ssh/eternus ${CINDER_LIB}
    ${CHOWN_CMD} cinder:cinder ${CINDER_LIB}/eternus
	CHECK_PYOPENSSL

    if [ ! -z "${inst_cfg}" ]; then
        if [ ! -z "${DB_USERNAME}" ]; then
            if [ ! -z "${DB_PASSWORD}" ]; then
                # Upgrade hostname information in the database.
                sh ./update_hostname.sh ${DB_USERNAME} ${DB_PASSWORD}
                if [ $? -ne 0 ]; then
                    PR "Failed to upgrade hostname information. returned non-zero exit status 1." "ERROR"
                else
                    PR 'Upgrade hostname information successfully.'
                fi
            else
                PR 'Database is not updated, please try to execute "sh update_hostname.sh" manually.' 'WARNING'
            fi
        else
            PR 'Database is not updated, please try to execute "sh update_hostname.sh" manually.' 'WARNING'
        fi
    else
        # Upgrade hostname information in the database.
        sh ./update_hostname.sh
        if [ $? -ne 0 ]; then
            PR "Failed to upgrade hostname information. returned non-zero exit status 1." "ERROR"
        else
            PR 'Upgrade hostname information successfully.'
        fi
    fi
}

# UPDATE
UPDATE_CONFIG()
{
	local inst_cfg=$1

	TITLE "Update Cinder Configuration"
	if [ -z "${inst_cfg}" ]; then
		UPDATE_CONFIG_MANUAL
	else
		UPDATE_CONFIG_SILENT "${inst_cfg}"
	fi
}

# UNINSTALL
UNINSTALL()
{
	local inst_cfg=$1

	TITLE "Uninstall Start"

	inst_dir=`FIND_INSTDIR 2>/dev/null`
	if [ $? -ne 0 ]; then
		PR "Cannot find directory for cinder volume drivers" "ERROR"
		exit 1
	fi

	DELETE_FILE ${inst_dir}
}
#---------------
# Main
#---------------
inst_cmd=$1
inst_cfg=$2
PR_PRODUCT_INFO

# find multi exec
find_multi_exec=`pgrep -fo $0`
if [ $$ != ${find_multi_exec} ]; then
	PR "Another user is running this installer."
	PR "The installation is ended."
	exit 1
fi

# check login user
inst_user=`${LS_CMD} -ld ${inst_dir}|${AWK_CMD} '{print $3}'`
if [ ${inst_user} != `${WHOAMI_CMD}` ]; then
	PR "Please execute installer after re-log in as a user \"${inst_user}\" where you installed cinder." "ERROR"
	exit 1
fi

case "${inst_cmd}" in
	"install")
		INSTALL "${inst_cfg}"
		;;
	"update")
		UPDATE_CONFIG "${inst_cfg}"
		;;
	"uninstall")
		UNINSTALL
		;;
	*)
		PR_ERROR_USAGE
		;;
esac

echo ""
PR "${inst_cmd^} is FINISHED"
PR "Please reboot cinder services to reflect configuration done by this installer."

exit 0