#!/bin/bash
#---------------
# Update Cinder.volumes database
#---------------

# yes or not
YES_NO()
{
    while true
    do
        echo -n "$1[yes/no]: "
        read -r answer
        case ${answer} in
            yes)
                return 0;;
            no)
                return 1;;
            *)
                echo "please input yes or no.";;
        esac
    done
}
echo ""
echo "====================================="
echo "  Upgrade cinder database information"
echo "====================================="

flag=""
user=$1
pass=$2
host=$3
if [ -z "${user}" ]; then
    skipsure=0
    echo 'If you do not know the username and password for accessing the database, '
    echo 'please check the [database] option in cinder.conf (eg: /etc/cinder/cinder.conf)'
    echo 'Such as: mysql+pymysql://cinder:abcd@controller/cinder'
    echo 'The username is "cinder" and password is "abcd" and host is "controller".'
    echo "====================================="
    echo "When upgrade from version 1.5.0 or earlier should update cinder database to change the volume's host name."
else
    skipsure=1
fi

while [ -z "${flag}" ]
do
    if [ ${skipsure} -eq 0 ]; then
        YES_NO 'Continue to update cinder database? '
        if [ $? -ne 0 ]; then
            echo 'Terminate the execution of update database, please try to execute "sh update_hostname.sh" manually.'
            exit 1
        fi
        if [ -z "${user}" ]; then
            while [ -z "${user}" ]
            do
                echo -n "username: "
                read -r user
            done
        fi
        if [ -z "${pass}" ]; then
            while [ -z "${pass}" ]
            do
                echo -n "password: "
                read -r pass
            done
        fi
		if [ -z "${host}" ]; then
            while [ -z "${host}" ]
            do
                echo -n "host: "
                read -r host
            done
        fi
    fi

    ret=`mysql -u${user} -p${pass} -h${host} -e "use cinder"`
    if [ $? -ne 0 ]; then
        if [ ${skipsure} -eq 1 ]; then
            echo 'ERROR: Database authentication failed.'
            echo 'Terminate the execution of update database, please try to execute "sh update_hostname.sh" manually.'
            exit 1
        fi
        echo 'ERROR: Authentication failed, please re-enter username, password and host.'
        flag=""
        user=""
        pass=""
	    host=""
        continue
    else
        flag="1"
        echo 'Authentication succeeded.'
    fi

    ret=`mysqldump -u${user} -p${pass} -h${host} cinder volumes > ./update_hostname_bk_cinder_volumes.sql`
    if [ $? -ne 0 ]; then
        echo 'ERROR: Backup database failed! No modification performed, exit.'
        exit 1
    else
        echo 'Successfully backed up the volumes table information. In "./update_hostname_bk_cinder_volumes.sql"'
    fi

    echo "Ready to execute:\"UPDATE cinder.volumes a, cinder.volume_metadata b SET a.host = CONCAT(SUBSTRING_INDEX(a.host, '#', 1), '#', b.value) WHERE a.host IS NOT NULL AND b.key = 'FJ_Pool_Name' AND a.id = b.volume_id\""

    ret=`mysql -u${user} -p${pass} -h${host} -e"UPDATE cinder.volumes a, cinder.volume_metadata b SET a.host = CONCAT(SUBSTRING_INDEX(a.host,'#',1),'#',b.value) WHERE a.host IS NOT NULL AND b.key='FJ_Pool_Name' AND a.id=b.volume_id"`
    if [ $? -eq 0 ]; then
        echo 'execution succeed.'
        exit 0
    else
        echo 'ERROR: Upgrading the database failed. Modify failed.'
        ret=`mysql -u${user} -p${pass} -h${host} cinder < ./update_hostname_bk_cinder_volumes.sql`
        if [ $? -ne 0 ]; then
            echo 'ERROR: Attempt to restore failed, exit.'
        else
            echo 'The "volumes" table has been restored, exit.'
        fi
        exit 1
    fi

done

exit 1